/* ***********************************************************************
   *  OpenGL - reseau :        construction rail                          *
   *                                                                     *
   *                                                                     *
   * objectif(s): construction d'un chemin de fer                        *
   * ------------------------------------------------------------------- *
   * version    : 1.00                                                   *
   * fichier    : OpenGL_main.c                                          *
   * ------------------------------------------------------------------- *
   * auteur(s)  : P. Chaudeyrac                                          *
   * ------------------------------------------------------------------- *
   * langage    : C++                                                    *
   * systeme    : Linux                                                  *
   * ------------------------------------------------------------------- *
   * historique :                                                        *
   *  10 08 2004  creation                                               *
   ***********************************************************************
*/

/* ******************************************************************** */
/*                     include                                          */
/* ******************************************************************** */

// fichier lie au TD 2
#include<OpenGL_rail.h>
#include<OpenGL_train.h>
#include<OpenGL_reseau.h>

// OpenGl
#include<GL/glut.h>

// C
#include<stdlib.h>
#include<stdio.h>
#include<math.h>

#include<sys/time.h>
#include<unistd.h>
#include <pthread.h>


/* ******************************************************************** */
/*       declaration des fonctions                                      */
/*                                                                      */
/* draw             : affichage de la scene                             */
/* drawAxes         : methode appellee par draw pour dessiner un axe    */
/*                    rouge vert bleu.                                  */
/* drawCube         : methode appellee par draw pour dessiner un cube   */
/*                    en fil de fer (ou presque...)                     */
/* init             : initialisation des parametres                     */
/* manageKeyboard   : cette fonction est appelle des qu'un utilisateur  */
/*                    appuie sur une touche du clavier.                 */
/* managePressMouse : cette fonction est appelee lorsqu'un utilisateur  */
/*                    presse un bouton de la souris.                    */
/* manageMoveMouse  : cette fonction est appelee lorsqu'un utilisateur  */
/*                    deplace la souris.                                */
/* resize           : cette fonction est appelee lorsqu'un utilisateur  */
/*                    redimensionne la fenetre. Cette fonction est      */
/*                    aussi appelee lors de la creation de la fentre.   */
/* ******************************************************************** */
void draw(void);
void drawAxes(void);
void init(void);
void initModelview();
void manageKeyboard(unsigned char key, int x, int y);
void managePressMouse(int button, int state, int x, int y);
void manageMoveMouse(int x, int y);
void resize(int widthWindows,int heightWindows);
void idle(void);
void setFeu(int res, int red);

/* ******************************************************************** */
/*                         quelques variables globales                  */
/*                                                                      */
/* alpha  : angle en degre exprimant la rotation suivant l'axe de z     */
/* ******************************************************************** */

/* pour la vue globale */
GLdouble alpha        = 160.0;
GLdouble phi          = 30.0;
GLdouble rho          = 10.0;

/* Rseaux de rails */
Reseau reseau;
Reseau reseau2;

/* ******************************************************************** */
/*                                main                                  */
/* ******************************************************************** */
int main(int argc,char **argv)
{
  /* initialisation de glut */
  glutInit(&argc,argv);
  glutInitDisplayMode(GLUT_DOUBLE|GLUT_RGB|GLUT_DEPTH);

  /* taille, position et nom (=nom de l'executable) de la fenetre */
  glutInitWindowSize(500,500);
  glutInitWindowPosition(100,100);
  glutCreateWindow(argv[0]);

  /* vos initialisations */
  init();

  /* association de vos fonctions aux evenements que glut gere */
  glutDisplayFunc(draw);
  glutReshapeFunc(resize);
  glutKeyboardFunc(manageKeyboard);
  glutMouseFunc(managePressMouse);
  glutMotionFunc(manageMoveMouse);
  glutIdleFunc(idle);

  /* lancement de la boucle de traitement de Glut */
  glutMainLoop();

  /* fin du programme */
  return 0;
}


void setFeu(int res, int red)
{
    if(res == 1)
    {
        if(red)
        {
           reseau.signalisation[4] = 0;
        }
        else
        {
            reseau.signalisation[4] = 1;
        }
    }
    else
    {
        if(red)
        {
            reseau2.signalisation[4] = 0;
        }
        else
        {
            reseau2.signalisation[4] = 1;
        }
    }

}


void idle(void)
{
    glutPostRedisplay();
}


/* ******************************************************************** */
/*                                draw                                  */
/* ******************************************************************** */
void draw(void)
{
  /* on vide les buffer */
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
  glMatrixMode(GL_MODELVIEW);

  /* on met la matrice dans la pile */
  glPushMatrix();

  /* on eloigne les objets de la camera */
  glTranslated(0.0,rho,0.0);

  /* on tourne autour de l'objet */
  glRotated(phi,1.0,0.0,0.0);
  glRotated(alpha,0.0,0.0,1.0);

  /* on dessine les axes */
  drawAxes();

  /* Dessin du rseau */
  drawReseau(&reseau);

  /* creation de la loco */
  drawTrain(&reseau);

  /* on remet l'ancienne matrice */
  glPopMatrix();

  /* on met a jour l'affiche dans la fenetre */
  glutSwapBuffers();
}


/* ******************************************************************** */
/*                                drawAxes                              */
/*                                                                      */
/* on dessine les axes segments par segments en associant une couleur   */
/* differente a chaque axe : l'axe des x en rouge RGB=(1,0,0)...        */
/* ******************************************************************** */
void drawAxes(void)
{
 /* on dessine les axes */
  glBegin(GL_LINES);
    glColor3d(1.0, 0.0, 0.0);
    glVertex3d(0.0, 0.0, 0.0); glVertex3d(1.0, 0.0, 0.0);

    glColor3d(0.0, 1.0, 0.0);
    glVertex3d(0.0, 0.0, 0.0); glVertex3d(0.0, 1.0, 0.0);

    glColor3d(0.0, 0.0, 1.0);
    glVertex3d(0.0, 0.0, 0.0); glVertex3d(0.0, 0.0, 1.0);
  glEnd();
}


/* ******************************************************************** */
/*                                init                                  */
/* ******************************************************************** */
void init(void)
{
  /* on active le z-buffer */
  glEnable(GL_DEPTH_TEST);
  glClearColor(0.3f,0.3f,0.3f,1.0f);
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

  /* taille du reseau */
  initReseau(&reseau);
  initReseau(&reseau2);
}


/* ******************************************************************** */
/*                            initModelview                             */
/* ******************************************************************** */
void initModelview()
{
  /* definition de la matrice modelview     */
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
  gluLookAt(0.0,  -2.0, 0.0,   /* coordonnees de la camera (=oeil)  */
	    0.0,   0.0, 0.0,   /* coordonnees du centre de la scene */
	    0.0,   0.0, 1.0);  /* vecteur designant l'axe de y      */
}


/* ******************************************************************** */
/*                             manageKeyboard                           */
/* ******************************************************************** */
void manageKeyboard(unsigned char key, int x, int y)
{
  switch (key)
  {
  case 27:             /* code de la touche echap */
    exit(0);
    break;

  case '2':
    phi+=5;
    break;

  case '8':
    phi-=5;
    break;

  case '4':
    alpha-=5;
    break;

  case '6':
    alpha+=5;
    break;
  case '-':
    if (rho>1) rho-=1;
    break;
  case '+':
    rho+=1;
    break;
  case '0':
    marcheAvant(&reseau,0);
    break;
  case '.':
    marcheArriere(&reseau,0);
    break;
  }
   /* on reaffiche = on force glut a rappeller a la fonction draw */
   glutPostRedisplay();
}


/* ******************************************************************** */
/*                             managePressMouse                         */
/* ******************************************************************** */
void managePressMouse(int button, int state, int x, int y)
{
}


/* ******************************************************************** */
/*                             manageMoveMouse                          */
/*                                                                      */
/* parametres :                                                         */
/*  x,y : les coordonnees du curseur dans la fentre                    */
/* ******************************************************************** */
void manageMoveMouse(int x, int y)
{
}


/* ******************************************************************** */
/*                                resize                                */
/* ******************************************************************** */
void resize(int widthWindows,int heightWindows)
{
  /* definition la taille de l'ecran OpenGl */
  /* ici la meme taille que notre fentre    */
  glViewport(0,0,widthWindows,heightWindows);

  /* definition de la matrice de projection */
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  gluPerspective(45.0f,widthWindows/heightWindows,0.1f,60.0f);

  /* definition de la matrice modelview     */
  initModelview();
}



