/* ***********************************************************************
   *  OpenGL - reseau :       construction des rails                     *
   * ------------------------------------------------------------------- *
   * fichier    : OpenGL_rail.c                                          *
   ***********************************************************************
*/


/* ******************************************************************** */
/*                            include                                   */
/* ******************************************************************** */
#include<OpenGL_rail.h>
#include<OpenGL_train.h>

#include<GL/glut.h>
#include<stdlib.h>
#include<stdio.h>
#include<math.h>


/* ******************************************************************** */
/*                      variables pour definir les rails                */
/* ******************************************************************** */
#define RAIL_RIGHT_LONG  1.0
#define RAIL_DEPTH       0.05
#define RAIL_HEIGHT      0.05
#define RAIL_RIGHT_BORD  0.3

#define TRAVERSE_LONG    0.8
#define TRAVERSE_WIDTH   0.1
#define TRAVERSE_HEIGHT  0.05
#define TRAVERSE_DORD    ((1.0-0.8)/2.0)
#define RAIL_CURVE_NB_SEGMENTS 5


/* ******************************************************************** */
/*                                drawRightRail                         */
/*                                                                      */
/* on dessine une portion de rail droite.                               */
/* ******************************************************************** */
void drawRightRail(void)
{
  static GLint railRightId=0;

  /* si l'objet n'est pas definit */
  if (railRightId==0)
  {
    /* on demande l'id d'un objet */
    railRightId = glGenLists(1);

    /* on definit l'objet en signalant qu'il sera pre calcule */
    glNewList(railRightId,GL_COMPILE);

    glColor3d(1.0, 1.0, 0.0);

    /* premiere rail */
    glPushMatrix();
    glTranslatef(0.5*RAIL_RIGHT_LONG,RAIL_RIGHT_BORD,0.5*RAIL_HEIGHT+TRAVERSE_HEIGHT);
    glScalef(RAIL_RIGHT_LONG,RAIL_DEPTH,RAIL_HEIGHT);
    glutSolidCube(1.0);
    glPopMatrix();

    /* deuxieme rail */
    glPushMatrix();
    glTranslatef(0.5*RAIL_RIGHT_LONG,RAIL_RIGHT_LONG-RAIL_RIGHT_BORD,0.5*RAIL_HEIGHT+TRAVERSE_HEIGHT);
    glScalef(RAIL_RIGHT_LONG,RAIL_DEPTH,RAIL_HEIGHT);
    glutSolidCube(1.0);
    glPopMatrix();

    /* traverse */
    glColor3d(0.7, 0.5, 0.2);
    glPushMatrix();
    glTranslatef(0.25,+TRAVERSE_DORD+0.5*TRAVERSE_LONG,0.5*TRAVERSE_HEIGHT);
    glScalef(TRAVERSE_WIDTH,TRAVERSE_LONG,TRAVERSE_HEIGHT);
    glutSolidCube(1.0);
    glPopMatrix();

    glPushMatrix();
    glTranslatef(0.75,+TRAVERSE_DORD+0.5*TRAVERSE_LONG,0.5*TRAVERSE_HEIGHT);
    glScalef(TRAVERSE_WIDTH,TRAVERSE_LONG,TRAVERSE_HEIGHT);
    glutSolidCube(1.0);
    glPopMatrix();

    glEndList();
  }

  /* pour l'afficher, on l'appelle seulement par son id */
  glCallList(railRightId);
}


/* ******************************************************************** */
/*                                drawRightCurveRail                    */
/*                                                                      */
/* on dessine une portion de rail droite.                               */
/* ******************************************************************** */
void drawRightCurveRail(void)
{
  static GLint railRightCurveId=0;
  int i;

  /* si l'objet n'est pas definit */
  if (railRightCurveId==0)
  {
    /* tableau des cosinus et des sinus */
    int    nbPoints = RAIL_CURVE_NB_SEGMENTS+1;
    double a=3.14/(2*RAIL_CURVE_NB_SEGMENTS);
    double c[RAIL_CURVE_NB_SEGMENTS+1];
    double s[RAIL_CURVE_NB_SEGMENTS+1];
    double rI,rE;

    /* initialisation de ces tableau d'angles */
    for(i=0;i<nbPoints;i++)
    {
      s[i]=sin(i*a);
      c[i]=cos(i*a);
    }

    /* on demande l'id d'un objet */
    railRightCurveId = glGenLists(1);

    /* on definit l'objet en signalant qu'il sera pre calcule */
    glNewList(railRightCurveId,GL_COMPILE);

    glColor3d(1.0, 1.0, 0.0);

    /* premiere rail */
    rI=RAIL_RIGHT_BORD-(RAIL_DEPTH/2.0);
    rE=RAIL_RIGHT_BORD+(RAIL_DEPTH/2.0);
    glPushMatrix();
    glTranslatef(0.0,0.0,TRAVERSE_HEIGHT);
    glBegin(GL_QUADS);
      for(i=0;i<(RAIL_CURVE_NB_SEGMENTS);i++)
      {
	glVertex3d(rI*c[i],  rI*s[i],0.0);
	glVertex3d(rI*c[i],  rI*s[i],RAIL_HEIGHT);
	glVertex3d(rI*c[i+1],rI*s[i+1],RAIL_HEIGHT);
	glVertex3d(rI*c[i+1],rI*s[i+1],0.0);

	glVertex3d(rE*c[i],  rE*s[i],0.0);
	glVertex3d(rE*c[i],  rE*s[i],RAIL_HEIGHT);
	glVertex3d(rE*c[i+1],rE*s[i+1],RAIL_HEIGHT);
	glVertex3d(rE*c[i+1],rE*s[i+1],0.0);

	/* partie superieure de la rail */
	glVertex3d(rE*c[i],  rE*s[i],  RAIL_HEIGHT);
	glVertex3d(rI*c[i],  rI*s[i],  RAIL_HEIGHT);
	glVertex3d(rI*c[i+1],rI*s[i+1],RAIL_HEIGHT);
	glVertex3d(rE*c[i+1],rE*s[i+1],RAIL_HEIGHT);

	/* partie inferieure de la rail */
	glVertex3d(rE*c[i],  rE*s[i],  RAIL_HEIGHT);
	glVertex3d(rI*c[i],  rI*s[i],  RAIL_HEIGHT);
	glVertex3d(rI*c[i+1],rI*s[i+1],RAIL_HEIGHT);
	glVertex3d(rE*c[i+1],rE*s[i+1],RAIL_HEIGHT);
      }
    glEnd();
    glPopMatrix();

    /* premiere rail */
    rI=1-RAIL_RIGHT_BORD+(RAIL_DEPTH/2.0);
    rE=1-RAIL_RIGHT_BORD-(RAIL_DEPTH/2.0);
    glPushMatrix();
    glTranslatef(0.0,0.0,TRAVERSE_HEIGHT);
    glBegin(GL_QUADS);
      for(i=0;i<(RAIL_CURVE_NB_SEGMENTS);i++)
      {
	glVertex3d(rI*c[i],  rI*s[i],0.0);
	glVertex3d(rI*c[i],  rI*s[i],RAIL_HEIGHT);
	glVertex3d(rI*c[i+1],rI*s[i+1],RAIL_HEIGHT);
	glVertex3d(rI*c[i+1],rI*s[i+1],0.0);

	glVertex3d(rE*c[i],  rE*s[i],0.0);
	glVertex3d(rE*c[i],  rE*s[i],RAIL_HEIGHT);
	glVertex3d(rE*c[i+1],rE*s[i+1],RAIL_HEIGHT);
	glVertex3d(rE*c[i+1],rE*s[i+1],0.0);

	/* partie superieure de la rail */
	glVertex3d(rE*c[i],  rE*s[i],  RAIL_HEIGHT);
	glVertex3d(rI*c[i],  rI*s[i],  RAIL_HEIGHT);
	glVertex3d(rI*c[i+1],rI*s[i+1],RAIL_HEIGHT);
	glVertex3d(rE*c[i+1],rE*s[i+1],RAIL_HEIGHT);

	/* partie inferieure de la rail */
	glVertex3d(rE*c[i],  rE*s[i],  RAIL_HEIGHT);
	glVertex3d(rI*c[i],  rI*s[i],  RAIL_HEIGHT);
	glVertex3d(rI*c[i+1],rI*s[i+1],RAIL_HEIGHT);
	glVertex3d(rE*c[i+1],rE*s[i+1],RAIL_HEIGHT);
      }
    glEnd();
    glPopMatrix();

    /* traverse */
    glColor3d(0.7, 0.5, 0.2);
    glPushMatrix();
    glRotated(30.0,0.0,0.0,1.0);
    glTranslatef(TRAVERSE_DORD+0.5*TRAVERSE_LONG,0.0,0.5*TRAVERSE_HEIGHT);
    glScalef(0.8*TRAVERSE_LONG,TRAVERSE_WIDTH,TRAVERSE_HEIGHT);
    glutSolidCube(1.0);
    glPopMatrix();

    /* traverse */
    glColor3d(0.7, 0.5, 0.2);
    glPushMatrix();
    glRotated(60.0,0.0,0.0,1.0);
    glTranslatef(TRAVERSE_DORD+0.5*TRAVERSE_LONG,0.0,0.5*TRAVERSE_HEIGHT);
    glScalef(0.8*TRAVERSE_LONG,TRAVERSE_WIDTH,TRAVERSE_HEIGHT);
    glutSolidCube(1.0);
    glPopMatrix();

    glEndList();
  }

  /* pour l'afficher, on l'appelle seulement par son id */
  glCallList(railRightCurveId);
}


void createRailNetwork(const int size, const int *data, const int *sign)
{
  int i;
  for(i=0;i<size;i++)
  {
    switch(data[i]) {
    case 0:
    {
      glTranslated(1.0,0,0);
      drawRightRail();
      break;
    }
    case 1:
    {
      glTranslated(1.0,0,0);
      drawRightCurveRail();
      break;
    }
    case 2:
    {
      glTranslated(1.0,1.0,0.0);
      glRotated(-90,0,0,1);
      glPushMatrix();
      glTranslated(1.0,0,0);
      glRotated(90,0,0,1);
      drawRightCurveRail();
      glPopMatrix();
      break;
    }
    }

    switch(sign[i]) {
    case 0:
    case 1:
        {
            glPushMatrix();
            glTranslated(1.0,0.0,0.0);
            drawSig(!sign[i]);
            glPopMatrix();
            break;
        }
    }
  }
}


//pour le feu de signalisation :
void drawSig(int signstop)
{

  if(signstop)
    glColor3f(1.0,0.0,0.0);
  else
    glColor3f(0.0,1.0,0.0);

  //draw a rail sign
  glPushMatrix();
  glRotatef(90.0,1.0,0.0,0.0);
  glPushMatrix();


  glTranslatef(0.0,0.965,-0.075);
  if(signstop)
    glRotatef(90,0.0,0.0,1.0);

  glPushMatrix();
  glTranslatef(0.0,0.0,-0.025);
  drawCylinder(0.025f, 0.05, 20, 20, 1, 2*M_PI);
  glPopMatrix();

  glPushMatrix();
  glTranslatef(0.0,-0.2,-0.025);
  drawCylinder(0.025f, 0.05, 20, 20, 1, 2*M_PI);
  glPopMatrix();

  glPushMatrix();
  glTranslatef(0.0,-0.1,0.0);
  glScalef(0.05f,0.2f,0.05f);
  glutSolidCube(1.0f);
  glPopMatrix();

  glPopMatrix();

  glPushMatrix();
  glTranslatef(0.0,0.5,0.0);
  glScalef(0.1f,1.0f,0.1f);
  glutSolidCube(1.0f);
  glPopMatrix();
  glPopMatrix();
}





