/* ***********************************************************************
   *  OpenGL - reseau :      construction du reseau                      *
   * ------------------------------------------------------------------- *
   * fichier    : OpenGL_reseau.c                                        *
   ***********************************************************************
*/


/* ******************************************************************** */
/*                            include                                   */
/* ******************************************************************** */
#include<OpenGL_reseau.h>
#include<OpenGL_rail.h>

#include<GL/glut.h>
#include<stdlib.h>
#include<stdio.h>
#include<math.h>
#include<string.h>

/* ******************************************************************** */
/*                      variables pour definir les rails                */
/* ******************************************************************** */
int NETWORK_01[] =          { 0, 0, 2, 0, 0, 0, 2, 0, 0, 0, 0, 0, 2,  0, 0, 0, 2, 0, 0, 0};
int SIGNALISATION_01[] =    {-1,-1,-1,-1, 1,-1,-1,-1,-1,-1,-1,-1,-1,112,-1,-1,-1,-1,-1,-1};



/* ******************************************************************** */
/*                                drawReseau                            */
/*                                                                      */
/* ******************************************************************** */
void drawReseau(Reseau *r)
{
  /* on dessine le chemin de fer */
  createRailNetwork(r->networkSize, r->network, r->signalisation);
}


void drawTrain(Reseau *r)
{
  /* creation de la loco */
  glPushMatrix();
  glTranslated(r->train.locoX,r->train.locoY,r->train.locoZ);
  glRotated(r->train.locoAlpha,0.0,0.0,1.0);
  drawLoco();
  glPopMatrix();
}



/* ******************************************************************** */
/*                                initReseau                            */
/*                                                                      */
/* ******************************************************************** */
void initReseau(Reseau *r)
{
  r->networkSize = sizeof(NETWORK_01)/sizeof(int);

  r->network=NETWORK_01;
  r->signalisation = (int*)malloc(r->networkSize*sizeof(int));
  memcpy(r->signalisation, SIGNALISATION_01, r->networkSize*sizeof(int));


  initTrain(&r->train);
  r->train.position=r->networkSize-1;
}



/* ******************************************************************** */
/*                        updatePosition                                */
/* ******************************************************************** */
void updatePosition(Reseau *r,int p)
{
  Train *t=&r->train;
  if(p<0) p+=r->networkSize;
  p%=r->networkSize;

  int tmp;
  if (t->locoSens==1) tmp=p;
  else tmp=t->position;

  if(r->network[tmp]==0)
  {
    if(t->locoAlpha==0) t->locoX+= t->locoSens * 1.0;
    else if(t->locoAlpha==-90) t->locoY-= t->locoSens * 1.0;
    else if(t->locoAlpha==-180) t->locoX-= t->locoSens * 1.0;
    else if(t->locoAlpha==-270) t->locoY+= t->locoSens * 1.0;
  }

  else if(r->network[tmp]==2)
  {
    if(t->locoAlpha==0)
    {
      if(t->locoSens==1)
      {
	t->locoX+= t->locoSens * 1.0;
	t->locoAlpha=-90;
      }
      else
      {
	t->locoY-=1.0;
	t->locoAlpha=-270;
      }
    }
    else if(t->locoAlpha==-90)
    {
      if(t->locoSens==1)
      {
	t->locoY-=1.0;
	t->locoAlpha=-180;
      }
      else
      {
	t->locoX-=1.0;
	t->locoAlpha+=90;
      }
    }
    else if(t->locoAlpha==-180)
    {
      if(t->locoSens==1)
      {
	t->locoX-=1.0;
	t->locoAlpha=-270;
      }
      else
      {
	t->locoY+=1.0;
	t->locoAlpha+=90;
      }
    }
    else if(t->locoAlpha==-270)
    {
      if(t->locoSens==1)
      {
	t->locoY+=1.0;
	t->locoAlpha=0;
      }
      else
      {
	t->locoX+=1.0;
	t->locoAlpha+=90;
      }
    }
  }

  /* mise a jour */
  t->position=p;
  t->locoSens=1;
}


void marcheAvant(Reseau *r, int idTrain)
{
  updatePosition(r,r->train.position+1);
}

void marcheArriere(Reseau *r, int idTrain)
{
  r->train.locoSens=-1;
  updatePosition(r,r->train.position-1);
}
