/* ***********************************************************************
   *  OpenGL - reseau :       construction du train                      *
   * ------------------------------------------------------------------- *                                 
   * fichier    : OpenGL_train.c                                         *
   ***********************************************************************
*/


/* ******************************************************************** */
/*                            include                                   */
/* ******************************************************************** */
#include<OpenGL_train.h>

#include<GL/glut.h>
#include<stdlib.h>
#include<stdio.h>
#include<math.h>


/* ******************************************************************** */
/*                      variables pour definir les rails                */
/* ******************************************************************** */
#define TRAIN_BODY_LONG     0.6
#define TRAIN_BODY_WIDTH    0.4
#define TRAIN_BODY_HEIGHT   0.2

#define TRAIN_CABIN_LONG    0.3
#define TRAIN_CABIN_WIDTH   0.4
#define TRAIN_CABIN_HEIGHT  0.6

#define TRAIN_CHIMNEY_WIDTH    0.1
#define TRAIN_CHIMNEY_HEIGHT   0.08

#define WHELL_INNER_RADIUS     0.01
#define WHELL_OUTER_RADIUS     0.1


/* ******************************************************************** */
/*                                drawTrain                             */
/*                                                                      */
/* ******************************************************************** */
void drawLoco(void)
{
  static GLint trainId=0;

  /* si l'objet n'est pas definit */
  if (trainId==0)
  {
    /* on demande l'id d'un objet */
    trainId = glGenLists(1);

    /* on definit l'objet en signalant qu'il sera pre calcule */
    glNewList(trainId,GL_COMPILE);

    glColor3d(0.0, 1.0, 0.0);

    /*                               */
    /* premiere bloc (bloc avant )   */
    /*                               */
    glPushMatrix();
    glTranslated(TRAIN_CABIN_LONG,0.0,TRAIN_BODY_HEIGHT/2.0);
    glPushMatrix();
    glTranslatef(TRAIN_BODY_LONG/2,0.0,0.0);
    glScalef(TRAIN_BODY_LONG,TRAIN_BODY_WIDTH,TRAIN_BODY_HEIGHT);
    glutSolidCube(1.0);
    glPopMatrix();

    /* cylindre du corps de la loco */
    glColor3d(1.0, 1.0, 1.0);
    glPushMatrix();
    /*    glTranslatef(0.5*RAIL_RIGHT_LONG,RAIL_RIGHT_BORD,0.5*RAIL_HEIGHT+TRAVERSE_HEIGHT);*/
    //    glScalef(TRAIN_BODY_LONG,TRAIN_BODY_WIDTH,TRAIN_BODY_HEIGHT);
    glTranslated(0.0,0.0,TRAIN_BODY_HEIGHT/2.0);
    glRotated(90,0.0,1.0,0.0);
    drawCylinder(TRAIN_BODY_WIDTH/2.0,TRAIN_BODY_LONG,10,10,1,2*M_PI);
    glPopMatrix();

    glPopMatrix(); 


    /*                               */
    /*                               */
    /*                               */
    glColor3d(0.0, 0.0, 1.0);
    glPushMatrix();
    glTranslated(TRAIN_CABIN_LONG+2*TRAIN_BODY_LONG/3.0,0.0,2*TRAIN_BODY_HEIGHT);
    drawCylinder(TRAIN_CHIMNEY_WIDTH/2.0,TRAIN_CHIMNEY_HEIGHT,10,10,1,2*M_PI);
    glPopMatrix();


    /*                               */
    /*           cabine              */
    /*                               */
    glPushMatrix();
    glTranslated(TRAIN_CABIN_LONG/2.0,0.0,TRAIN_CABIN_HEIGHT/2.0);
    glScalef(TRAIN_CABIN_LONG,TRAIN_CABIN_WIDTH,TRAIN_CABIN_HEIGHT);
    glutSolidCube(1.0);
    glPopMatrix();

    /*                               */
    /*           whells              */
    /*                               */
    glColor3d(0.0, 0.6, 0.8);
    glPushMatrix();
    glTranslated(TRAIN_CABIN_WIDTH/2.0,TRAIN_CABIN_WIDTH/2,0.0);
    drawWheel();
    glTranslated(TRAIN_BODY_WIDTH,0.0,0.0);
    drawWheel();
    glPopMatrix();

    glPushMatrix();
    glTranslated(TRAIN_CABIN_WIDTH/2.0,-TRAIN_CABIN_WIDTH/2,0.0);
    drawWheel();
    glTranslated(TRAIN_BODY_WIDTH,0.0,0.0);
    drawWheel();
    glPopMatrix();

    glEndList();
  }
  
  /* pour l'afficher, on l'appelle seulement par son id */
  glCallList(trainId);
}


void drawWheel(void)
{
  glPushMatrix();
  glRotated(90,1.0,0.0,0.0);
  glutSolidTorus(WHELL_INNER_RADIUS,WHELL_OUTER_RADIUS,20,20);
  glPopMatrix();
}


/* ******************************************************************** */
/*                            glutSolidCylinder                         */
/*                                                                      */
/* note pour le TD 2 : il n'est pas necessaire de comprendre cette      */
/*                     on reviendra dessus dans un TD ulterieur.        */
/* ******************************************************************** */
void drawCylinder(float r, float h, int dx, int dy, short fill, double angle)
{
  int ix,iy;
  double a;
  double x,y;

  /// The angle between each vertex around the cylinder
  double stepa = angle / (double)dx;
  
  /// The lenght between each vertex in th height part of the cylinder  
  double steph = h / (double)dy;

  /// draw the body of the cylinder
  for (iy = 0; iy < dy; ++iy){ 
    glBegin(GL_QUAD_STRIP);
    for (ix = 0, a = 0.0; ix <= dx; ++ix, a += stepa){
      x = r*cos(a);
      y = r*sin(a);
      glNormal3f( x, y, 0.0f);
      glVertex3f( x, y, (iy + 1) * steph);
      glVertex3f( x, y, iy * steph);
    } 
    glEnd();   
  }

  if(fill){ 
    glNormal3f( 0.0f, 0.0f, -1.0f);
    glBegin(GL_TRIANGLE_FAN);
    glVertex3f(0.0, 0.0, 0.0);    
    for (ix = 0, a = angle; ix <= dx; ++ix, a -= stepa){
      x = r*cos(a);
      y = r*sin(a);
      glVertex3f( x, y, 0.0f);
    } 
    glEnd();   
    glNormal3f( 0.0f, 0.0f, 1.0f);
    glBegin(GL_TRIANGLE_FAN);
    glVertex3f( 0.0f, 0.0f, h);   
    for ( ix = 0, a = 0.0; ix <= dx; ++ix, a += stepa){
      x = r*cos(a);
      y = r*sin(a);
      glVertex3f( x, y, h);
    } 
    glEnd();   
  }
}


void initTrain(Train *t)
{
  t->locoX     = 0.5;
  t->locoY     = 0.5;
  t->locoZ     = 0.2; 
  t->locoAlpha = 0 ;
  t->locoSens  = 1 ;
  t->position=0;
}
